/****************************************************************************
**
** Copyright (C) 2004-2005 Trolltech AS. All rights reserved.
**
** This file is part of the example classes of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include <QtGui>
#include <QtNetwork>

#include "ftpwindow.h"

FtpWindow::FtpWindow(MainForm *parent)
    : QDialog(parent), ftp(0)
{
    setupUi(this);
    p=parent;
    data=p->data;
    progressDialog = new QProgressDialog(this);

    connect(fileList, SIGNAL(itemDoubleClicked(QListWidgetItem *)),
            this, SLOT(processItem(QListWidgetItem *)));
    connect(fileList, SIGNAL(itemSelectionChanged()),
            this, SLOT(enableDownloadButton()));
    connect(progressDialog, SIGNAL(canceled()), this, SLOT(cancelDownload()));
    connect(connectButton, SIGNAL(clicked()), this, SLOT(connectOrDisconnect()));
    connect(cdToParentButton, SIGNAL(clicked()), this, SLOT(cdToParent()));
    connect(downloadButton, SIGNAL(clicked()), this, SLOT(downloadFiles()));
    connect(quitButton, SIGNAL(clicked()), this, SLOT(close()));

    ok=false;
    setWindowTitle(tr("FTP"));
    format=FFORMAT_SIMPSON;
}

void FtpWindow::connectOrDisconnect()
{
    if (ftp) {
        ftp->abort();
        ftp->deleteLater();
        fileList->setEnabled(false);
        downloadButton->setEnabled(false);
	formatComboBox->setEnabled(false);
        connectButton->setText(tr("Connect"));
        return;
    }
    
    QApplication::setOverrideCursor(Qt::WaitCursor);
    
    ftp = new QFtp(this);
    connect(ftp, SIGNAL(commandFinished(int, bool)),
            this, SLOT(ftpCommandFinished(int, bool)));
    connect(ftp, SIGNAL(listInfo(const QUrlInfo &)),
            this, SLOT(addToList(const QUrlInfo &)));
    connect(ftp, SIGNAL(dataTransferProgress(qint64, qint64)),
            this, SLOT(updateDataTransferProgress(qint64, qint64)));
    if (passiveCheckBox->isChecked()) 
	ftp->setTransferMode(QFtp::Passive);
    else
	ftp->setTransferMode(QFtp::Active);
    ftp->connectToHost(ftpServerLineEdit->text());
    ftp->login(loginLineEdit->text(),passwordLineEdit->text());
    ftp->list();

    fileList->setEnabled(true);
    connectButton->setText(tr("Disconnect"));
    statusLabel->setText(tr("Connecting to FTP server %1...")
                         .arg(ftpServerLineEdit->text()));
}

void FtpWindow::addToAllFiles(QString name){
	if (listContains(name))
			allFiles<<name;
}

void FtpWindow::downloadFiles()
{
	QString s=QString(QDir::homePath()+"/.gsim");	
	QDir tdir(s);
	bool ok;
	if (!tdir.exists()) {
		ok=tdir.mkpath(s);
		}
	tempdir=tdir.canonicalPath();
	if (!tempdir.endsWith("/"))
		tempdir+=QString("/");
	switch (formatComboBox->currentIndex()) {
	case 0:
		format=FFORMAT_SPINSIGHT;
		tempdir+=QString("temp_spinsight/");
		tdir.mkdir(tempdir); //create temp directory if it is not exist
		addToAllFiles("data");
		addToAllFiles("acq");
		addToAllFiles("proc");
		addToAllFiles("proc_setup");
		break;
	case 1:
		format=FFORMAT_XWINNMR;
		tempdir+=QString("temp_xwinnmr/");
		tdir.mkdir(tempdir); //create temp directory if it is not exist
		tempdir+=QString("1/");
		tdir.mkdir(tempdir);
		//create extra directories
//		t2+=QString("pdata/");
		tdir.mkdir(tempdir+QString("pdata/"));
//		t2+=QString("1/");
		tdir.mkdir(tempdir+QString("pdata/")+QString("1/"));
		addToAllFiles("fid");
		addToAllFiles("ser");
		addToAllFiles("acqu");
		addToAllFiles("acqu2");
		addToAllFiles("vdlist");
		addToAllFiles("vplist");
		addToAllFiles("vtlist");
		addToAllFiles("valist");
		break;
	case 2:
		format=FFORMAT_VNMR;
		tempdir+=QString("temp_vnmr/");
		tdir.mkdir(tempdir); //create temp directory if it is not exist
//		allFiles<<"fid"<<"procpar"<<"text";
		addToAllFiles("fid");
		addToAllFiles("procpar");
		addToAllFiles("text");
		break;
	}
	cout<<"Created dir:"<<tempdir.toAscii().data()<<endl;
	tdir.cd(tempdir);
//	tempdir.remove(QString("*"));
//	QString fileName = fileList->currentItem()->text();
	QString fileName=allFiles.takeFirst();
	mainfilename=tempdir+fileName;
	downloadFile(fileName);
}

bool FtpWindow::listContains(QString file)
{
	bool ok=false;
	int n=fileList->count();
	if (n<1)
		return false;
	for (size_t i=0; i<n; i++)
		if (fileList->item(i)->text()==file)
			ok=true;
	return ok;
}

void FtpWindow::downloadFile(QString fileName)
{
    cout<<"Start downloads\n";
    QString saveName = tempdir+fileName;

    if (QFile::exists(saveName)) {
 	QFile::remove(saveName);
    }

    file = new QFile(saveName);
    if (!file->open(QIODevice::WriteOnly)) {
        QMessageBox::information(this, tr("FTP"),
                                 tr("Unable to save the file %1: %2.")
                                 .arg(fileName).arg(file->errorString()));
        delete file;
        return;
    }
    cout<<"start downloading "<<fileName.toAscii().data()<<endl;
    ftp->get(fileName, file);
    cout<<"end downloading "<<fileName.toAscii().data()<<endl;

    progressDialog->setLabelText(tr("Downloading %1...").arg(fileName));
    progressDialog->setValue(0);
    progressDialog->show();
    downloadButton->setEnabled(false);
    formatComboBox->setEnabled(false);
}

void FtpWindow::cancelDownload()
{
    ftp->abort();
}

void FtpWindow::ftpCommandFinished(int, bool error)
{
    if (ftp->currentCommand() == QFtp::ConnectToHost) {
        if (error) {
            QApplication::restoreOverrideCursor();
            QMessageBox::information(this, tr("FTP"),
                                     tr("Unable to connect to the FTP server "
                                        "at %1. Please check that the host "
                                        "name is correct.")
                                     .arg(ftpServerLineEdit->text()));
            return;
        }

        statusLabel->setText(tr("Logged onto %1.")
                             .arg(ftpServerLineEdit->text()));
        fileList->setFocus();
        downloadButton->setDefault(true);
        return;
    }

    else if (ftp->currentCommand() == QFtp::Get) {
        QApplication::restoreOverrideCursor();
        if (error) {
	    cout<<"File"<<file->fileName().toAscii().data()<<" is not downloadable\n";
            statusLabel->setText(tr("Canceled download of %1.")
                                 .arg(file->fileName()));
            file->close();
            file->remove();
        } else {
            statusLabel->setText(tr("Downloaded %1 to current directory.")
                                 .arg(file->fileName()));
            file->close();
        }
	delete file;

	if (!allFiles.isEmpty()) {
			QString nextfile=allFiles.takeFirst();
			downloadFile(nextfile);
	}
	else {
		ok=true;
        	enableDownloadButton();
		close();
	}
    }
    else if (ftp->currentCommand() == QFtp::List) {
        QApplication::restoreOverrideCursor();
        if (isDirectory.isEmpty()) {
            fileList->addItem(tr("<empty>"));
            fileList->setEnabled(false);
        }
    }
}

void FtpWindow::addToList(const QUrlInfo &urlInfo)
{
    QListWidgetItem *item = new QListWidgetItem;
    item->setText(urlInfo.name());
    QPixmap pixmap(urlInfo.isDir() ? ":/images/dir.png" : ":/images/file.png");
    item->setIcon(pixmap);

    isDirectory[urlInfo.name()] = urlInfo.isDir();
    fileList->addItem(item);
    if (!fileList->currentItem()) {
        fileList->setCurrentItem(fileList->item(0));
        fileList->setEnabled(true);
    }
}

void FtpWindow::processItem(QListWidgetItem *item)
{
    QString name = item->text();
    if (isDirectory.value(name)) {
        fileList->clear();
        isDirectory.clear();
        currentPath += "/" + name;
        ftp->cd(name);
        ftp->list();
        cdToParentButton->setEnabled(true);
        QApplication::setOverrideCursor(Qt::WaitCursor);
        return;
    }
}

void FtpWindow::cdToParent()
{
    QApplication::setOverrideCursor(Qt::WaitCursor);
    fileList->clear();
    isDirectory.clear();
    currentPath = currentPath.left(currentPath.lastIndexOf('/'));
    if (currentPath.isEmpty()) {
        cdToParentButton->setEnabled(false);
        ftp->cd("/");
    } else {
        ftp->cd(currentPath);
    }
    ftp->list();
}

void FtpWindow::updateDataTransferProgress(qint64 readBytes,
                                           qint64 totalBytes)
{
   progressDialog->setMaximum(totalBytes);
   progressDialog->setValue(readBytes);
}

void FtpWindow::enableDownloadButton()
{
    QListWidgetItem *current = fileList->currentItem();
    if (current) {
        QString currentFile = current->text();
        downloadButton->setEnabled(!isDirectory.value(currentFile));
    } else {
        downloadButton->setEnabled(false);
    }
    formatComboBox->setEnabled(true);
}
